@@ -1,10 +1,10 @@
 """Commons module."""
 from splitio.util.time import get_current_epoch_time_ms
+from splitio.spec import SPEC_VERSION
 
 _CACHE_CONTROL = 'Cache-Control'
 _CACHE_CONTROL_NO_CACHE = 'no-cache'
 
-
 def headers_from_metadata(sdk_metadata, client_key=None):
     """
     Generate a dict with headers required by data-recording API endpoints.
@@ -57,7 +57,7 @@ def record_telemetry(status_code, elapsed, metric_name, telemetry_runtime_produc
 class FetchOptions(object):
     """Fetch Options object."""
 
-    def __init__(self, cache_control_headers=False, change_number=None):
+    def __init__(self, cache_control_headers=False, change_number=None, rbs_change_number=None, sets=None, spec=SPEC_VERSION):
         """
         Class constructor.
 
@@ -66,9 +66,15 @@ class FetchOptions(object):
 
         :param change_number: ChangeNumber to use for bypassing CDN in request.
         :type change_number: int
+
+        :param sets: list of flag sets
+        :type sets: list
         """
         self._cache_control_headers = cache_control_headers
         self._change_number = change_number
+        self._rbs_change_number = rbs_change_number
+        self._sets = sets
+        self._spec = spec
 
     @property
     def cache_control_headers(self):
@@ -80,16 +86,42 @@ class FetchOptions(object):
         """Return change number."""
         return self._change_number
 
+    @property
+    def rbs_change_number(self):
+        """Return change number."""
+        return self._rbs_change_number
+
+    @property
+    def sets(self):
+        """Return sets."""
+        return self._sets
+
+    @property
+    def spec(self):
+        """Return sets."""
+        return self._spec
+
     def __eq__(self, other):
         """Match between other options."""
         if self._cache_control_headers != other._cache_control_headers:
             return False
+
         if self._change_number != other._change_number:
             return False
+
+        if self._rbs_change_number != other._rbs_change_number:
+            return False
+
+        if self._sets != other._sets:
+            return False
+        
+        if self._spec != other._spec:
+            return False
+        
         return True
 
 
-def build_fetch(change_number, fetch_options, metadata):
+def build_fetch(change_number, fetch_options, metadata, rbs_change_number=None):
     """
     Build fetch with new flags if that is the case.
 
@@ -102,15 +134,24 @@ def build_fetch(change_number, fetch_options, metadata):
     :param metadata: Metadata Headers.
     :type metadata: dict
 
+    :param rbs_change_number: Last known timestamp of a rule based segment modification.
+    :type rbs_change_number: int
+
     :return: Objects for fetch
     :rtype: dict, dict
     """
-    query = {'since': change_number}
+    query = {'s': fetch_options.spec} if fetch_options.spec is not None else {}
+    query['since'] = change_number
+    if rbs_change_number is not None:
+        query['rbSince'] = rbs_change_number
     extra_headers = metadata
     if fetch_options is None:
         return query, extra_headers
+
     if fetch_options.cache_control_headers:
         extra_headers[_CACHE_CONTROL] = _CACHE_CONTROL_NO_CACHE
+    if fetch_options.sets is not None:
+        query['sets'] = fetch_options.sets
     if fetch_options.change_number is not None:
         query['till'] = fetch_options.change_number
     return query, extra_headers
\ No newline at end of file
