@@ -3,10 +3,10 @@ from numbers import Number
 import logging
 import re
 import math
+import inspect
 
-from splitio.api import APIException
-from splitio.api.commons import FetchOptions
 from splitio.client.key import Key
+from splitio.client import client
 from splitio.engine.evaluator import CONTROL
 
 
@@ -14,6 +14,7 @@ _LOGGER = logging.getLogger(__name__)
 MAX_LENGTH = 250
 EVENT_TYPE_PATTERN = r'^[a-zA-Z0-9][-_.:a-zA-Z0-9]{0,79}$'
 MAX_PROPERTIES_LENGTH_BYTES = 32768
+_FLAG_SETS_REGEX = '^[a-z0-9][_a-z0-9]{0,49}$'
 
 
 def _check_not_null(value, name, operation):
@@ -33,6 +34,7 @@ def _check_not_null(value, name, operation):
         _LOGGER.error('%s: you passed a null %s, %s must be a non-empty string.',
                       operation, name, name)
         return False
+
     return True
 
 
@@ -55,6 +57,7 @@ def _check_is_string(value, name, operation):
             operation, name, name
         )
         return False
+
     return True
 
 
@@ -75,10 +78,11 @@ def _check_string_not_empty(value, name, operation):
         _LOGGER.error('%s: you passed an empty %s, %s must be a non-empty string.',
                       operation, name, name)
         return False
+
     return True
 
 
-def _check_string_matches(value, operation, pattern):
+def _check_string_matches(value, operation, pattern, name, length):
     """
     Check if value is adhere to a regular expression passed.
 
@@ -91,16 +95,17 @@ def _check_string_matches(value, operation, pattern):
     :return: The result of validation
     :rtype: True|False
     """
-    if not re.match(pattern, value):
+    if re.search(pattern, value) is None or re.search(pattern, value).group() != value:
         _LOGGER.error(
-            '%s: you passed %s, event_type must ' +
+            '%s: you passed %s, %s must ' +
             'adhere to the regular expression %s. ' +
-            'This means an event name must be alphanumeric, cannot be more ' +
-            'than 80 characters long, and can only include a dash, underscore, ' +
+            'This means %s must be alphanumeric, cannot be more ' +
+            'than %s characters long, and can only include a dash, underscore, ' +
             'period, or colon as separators of alphanumeric characters.',
-            operation, value, pattern
+            operation, value, name, pattern, name, length
         )
         return False
+
     return True
 
 
@@ -119,6 +124,7 @@ def _check_can_convert(value, name, operation):
     """
     if isinstance(value, str):
         return value
+
     else:
         # check whether if isnan and isinf are really necessary
         if isinstance(value, bool) or (not isinstance(value, Number)) or math.isnan(value) \
@@ -126,6 +132,7 @@ def _check_can_convert(value, name, operation):
             _LOGGER.error('%s: you passed an invalid %s, %s must be a non-empty string.',
                           operation, name, name)
             return None
+
     _LOGGER.warning('%s: %s %s is not of type string, converting.',
                     operation, name, value)
     return str(value)
@@ -148,6 +155,7 @@ def _check_valid_length(value, name, operation):
         _LOGGER.error('%s: %s too long - must be %s characters or less.',
                       operation, name, MAX_LENGTH)
         return False
+
     return True
 
 
@@ -164,21 +172,21 @@ def _check_valid_object_key(key, name, operation):
     :return: The result of validation
     :rtype: str|None
     """
-    if key is None:
-        _LOGGER.error(
-            '%s: you passed a null %s, %s must be a non-empty string.',
-            operation, name, name)
+    if not _check_not_null(key, name, operation):
         return None
+
     if isinstance(key, str):
         if not _check_string_not_empty(key, name, operation):
             return None
+
     key_str = _check_can_convert(key, name, operation)
     if key_str is None or not _check_valid_length(key_str, name, operation):
         return None
+
     return key_str
 
 
-def _remove_empty_spaces(value, operation):
+def _remove_empty_spaces(value, name, operation):
     """
     Check if an string has whitespaces.
 
@@ -191,9 +199,15 @@ def _remove_empty_spaces(value, operation):
     """
     strip_value = value.strip()
     if value != strip_value:
-        _LOGGER.warning("%s: feature_name '%s' has extra whitespace, trimming.", operation, value)
+        _LOGGER.warning("%s: %s '%s' has extra whitespace, trimming.", operation, name, value)
     return strip_value
 
+def _convert_str_to_lower(value, name, operation):
+    lower_value = value.lower()
+    if value != lower_value:
+        _LOGGER.warning("%s: %s '%s' should be all lowercase - converting string to lowercase", operation, name, value)
+    return lower_value
+
 
 def validate_key(key, method_name):
     """
@@ -210,18 +224,19 @@ def validate_key(key, method_name):
     """
     matching_key_result = None
     bucketing_key_result = None
-    if key is None:
-        _LOGGER.error('%s: you passed a null key, key must be a non-empty string.', method_name)
+    if not _check_not_null(key, 'key', method_name):
         return None, None
 
     if isinstance(key, Key):
         matching_key_result = _check_valid_object_key(key.matching_key, 'matching_key', method_name)
         if matching_key_result is None:
             return None, None
+
         bucketing_key_result = _check_valid_object_key(key.bucketing_key, 'bucketing_key',
                                                        method_name)
         if bucketing_key_result is None:
             return None, None
+
     else:
         key_str = _check_can_convert(key, 'key', method_name)
         if key_str is not None and \
@@ -231,31 +246,28 @@ def validate_key(key, method_name):
     return matching_key_result, bucketing_key_result
 
 
-def validate_feature_name(feature_name, should_validate_existance, split_storage, method_name):
-    """
-    Check if feature_name is valid for get_treatment.
+def _validate_feature_flag_name(feature_flag_name, method_name):
+    if (not _check_not_null(feature_flag_name, 'feature_flag_name', method_name)) or \
+       (not _check_is_string(feature_flag_name, 'feature_flag_name', method_name)) or \
+       (not _check_string_not_empty(feature_flag_name, 'feature_flag_name', method_name)):
+        return False
 
-    :param feature_name: feature_name to be checked
-    :type feature_name: str
-    :return: feature_name
+    return True
+
+
+def validate_feature_flag_name(feature_flag_name, method_name):
+    """
+    Check if feature flag name is valid for get_treatment.
+
+    :param feature_flag_name: feature flag name to be checked
+    :type feature_flag_name: str
+    :return: feature_flag_name
     :rtype: str|None
     """
-    if (not _check_not_null(feature_name, 'feature_name', method_name)) or \
-       (not _check_is_string(feature_name, 'feature_name', method_name)) or \
-       (not _check_string_not_empty(feature_name, 'feature_name', method_name)):
+    if not _validate_feature_flag_name(feature_flag_name, method_name):
         return None
 
-    if should_validate_existance and split_storage.get(feature_name) is None:
-        _LOGGER.warning(
-            "%s: you passed \"%s\" that does not exist in this environment, "
-            "please double check what Splits exist in the web console.",
-            method_name,
-            feature_name
-        )
-        return None
-
-    return _remove_empty_spaces(feature_name, method_name)
-
+    return _remove_empty_spaces(feature_flag_name, 'feature flag name', method_name)
 
 def validate_track_key(key):
     """
@@ -268,41 +280,76 @@ def validate_track_key(key):
     """
     if not _check_not_null(key, 'key', 'track'):
         return None
+
     key_str = _check_can_convert(key, 'key', 'track')
     if key_str is None or \
        (not _check_string_not_empty(key_str, 'key', 'track')) or \
        (not _check_valid_length(key_str, 'key', 'track')):
         return None
+
     return key_str
 
 
-def validate_traffic_type(traffic_type, should_validate_existance, split_storage):
+def _validate_traffic_type_value(traffic_type):
+    if (not _check_not_null(traffic_type, 'traffic_type', 'track')) or \
+       (not _check_is_string(traffic_type, 'traffic_type', 'track')) or \
+       (not _check_string_not_empty(traffic_type, 'traffic_type', 'track')):
+        return False
+
+    return True
+
+def validate_traffic_type(traffic_type, should_validate_existance, feature_flag_storage):
     """
     Check if traffic_type is valid for track.
 
     :param traffic_type: traffic_type to be checked
     :type traffic_type: str
-    :param should_validate_existance: Whether to check for existante in the split storage.
+    :param should_validate_existance: Whether to check for existante in the feature flag storage.
     :type should_validate_existance: bool
-    :param split_storage: Split storage.
-    :param split_storage: splitio.storages.SplitStorage
+    :param feature_flag_storage: Feature flag storage.
+    :param feature_flag_storage: splitio.storages.SplitStorage
     :return: traffic_type
     :rtype: str|None
     """
-    if (not _check_not_null(traffic_type, 'traffic_type', 'track')) or \
-       (not _check_is_string(traffic_type, 'traffic_type', 'track')) or \
-       (not _check_string_not_empty(traffic_type, 'traffic_type', 'track')):
+    if not _validate_traffic_type_value(traffic_type):
         return None
-    if not traffic_type.islower():
-        _LOGGER.warning('track: %s should be all lowercase - converting string to lowercase.',
-                        traffic_type)
-        traffic_type = traffic_type.lower()
 
-    if should_validate_existance and not split_storage.is_valid_traffic_type(traffic_type):
+    traffic_type = _convert_str_to_lower(traffic_type, 'traffic type', 'track')
+
+    if should_validate_existance and not feature_flag_storage.is_valid_traffic_type(traffic_type):
         _LOGGER.warning(
-            'track: Traffic Type %s does not have any corresponding Splits in this environment, '
+            'track: Traffic Type %s does not have any corresponding Feature flags in this environment, '
             'make sure you\'re tracking your events to a valid traffic type defined '
-            'in the Split console.',
+            'in the Split user interface.',
+            traffic_type
+        )
+
+    return traffic_type
+
+
+async def validate_traffic_type_async(traffic_type, should_validate_existance, feature_flag_storage):
+    """
+    Check if traffic_type is valid for track.
+
+    :param traffic_type: traffic_type to be checked
+    :type traffic_type: str
+    :param should_validate_existance: Whether to check for existante in the feature flag storage.
+    :type should_validate_existance: bool
+    :param feature_flag_storage: Feature flag storage.
+    :param feature_flag_storage: splitio.storages.SplitStorage
+    :return: traffic_type
+    :rtype: str|None
+    """
+    if not _validate_traffic_type_value(traffic_type):
+        return None
+
+    traffic_type = _convert_str_to_lower(traffic_type, 'traffic type', 'track')
+
+    if should_validate_existance and not await feature_flag_storage.is_valid_traffic_type(traffic_type):
+        _LOGGER.warning(
+            'track: Traffic Type %s does not have any corresponding Feature flags in this environment, '
+            'make sure you\'re tracking your events to a valid traffic type defined '
+            'in the Split user interface.',
             traffic_type
         )
 
@@ -321,8 +368,9 @@ def validate_event_type(event_type):
     if (not _check_not_null(event_type, 'event_type', 'track')) or \
        (not _check_is_string(event_type, 'event_type', 'track')) or \
        (not _check_string_not_empty(event_type, 'event_type', 'track')) or \
-       (not _check_string_matches(event_type, 'track', EVENT_TYPE_PATTERN)):
+       (not _check_string_matches(event_type, 'track', EVENT_TYPE_PATTERN, 'an event name', 80)):
         return None
+
     return event_type
 
 
@@ -337,91 +385,138 @@ def validate_value(value):
     """
     if value is None:
         return None
+
     if (not isinstance(value, Number)) or isinstance(value, bool):
         _LOGGER.error('track: value must be a number.')
         return False
+
     return value
 
-
-def validate_manager_feature_name(feature_name, should_validate_existance, split_storage):
+def validate_manager_feature_flag_name(feature_flag_name, should_validate_existance, feature_flag_storage):
     """
-    Check if feature_name is valid for track.
+    Check if feature flag name is valid for track.
 
-    :param feature_name: feature_name to be checked
-    :type feature_name: str
-    :return: feature_name
+    :param feature_flag_name: feature flag name to be checked
+    :type feature_flag_name: str
+    :return: feature_flag_name
     :rtype: str|None
     """
-    if (not _check_not_null(feature_name, 'feature_name', 'split')) or \
-       (not _check_is_string(feature_name, 'feature_name', 'split')) or \
-       (not _check_string_not_empty(feature_name, 'feature_name', 'split')):
+    if not _validate_feature_flag_name(feature_flag_name, 'split'):
         return None
 
-    if should_validate_existance and split_storage.get(feature_name) is None:
+    feature_flag = feature_flag_storage.get(feature_flag_name)
+    if should_validate_existance and feature_flag is None:
         _LOGGER.warning(
             "split: you passed \"%s\" that does not exist in this environment, "
-            "please double check what Splits exist in the web console.",
-            feature_name
+            "please double check what Feature flags exist in the Split user interface.",
+            feature_flag_name
         )
         return None
 
-    return feature_name
+    return feature_flag
 
-
-def validate_features_get_treatments(  # pylint: disable=invalid-name
-    method_name,
-    features,
-    should_validate_existance=False,
-    split_storage=None
-):
+async def validate_manager_feature_flag_name_async(feature_flag_name, should_validate_existance, feature_flag_storage):
     """
-    Check if features is valid for get_treatments.
+    Check if feature flag name is valid for track.
 
-    :param features: array of features
-    :type features: list
-    :return: filtered_features
+    :param feature_flag_name: feature flag name to be checked
+    :type feature_flag_name: str
+    :return: feature_flag_name
+    :rtype: str|None
+    """
+    if not _validate_feature_flag_name(feature_flag_name, 'split'):
+        return None
+
+    feature_flag = await feature_flag_storage.get(feature_flag_name)
+    if should_validate_existance and feature_flag is None:
+        _LOGGER.warning(
+            "split: you passed \"%s\" that does not exist in this environment, "
+            "please double check what Feature flags exist in the Split user interface.",
+            feature_flag_name
+        )
+        return None
+
+    return feature_flag
+
+def validate_feature_flag_names(feature_flags, method_name):
+    """
+    Check if feature flag name is valid for track.
+
+    :param feature_flag_name: feature flag name to be checked
+    :type feature_flag_name: str
+    """
+    for feature_flag in  feature_flags.keys():
+        if feature_flags[feature_flag] is None:
+            _LOGGER.warning(
+                "%s: you passed \"%s\" that does not exist in this environment, "
+                "please double check what Feature flags exist in the Split user interface.",
+                method_name, feature_flag
+            )
+
+def _check_feature_flag_instance(feature_flags, method_name):
+    if feature_flags is None or not isinstance(feature_flags, list):
+        _LOGGER.error("%s: feature flag names must be a non-empty array.", method_name)
+        return False
+
+    if not feature_flags:
+        _LOGGER.error("%s: feature flag names must be a non-empty array.", method_name)
+        return False
+
+    return True
+
+
+def _get_filtered_feature_flag(feature_flags, method_name):
+    return set(
+        _remove_empty_spaces(feature_flag, 'feature flag name', method_name) for feature_flag in feature_flags
+        if feature_flag is not None and
+        _check_is_string(feature_flag, 'feature flag name', method_name) and
+        _check_string_not_empty(feature_flag, 'feature flag name', method_name)
+    )
+
+
+def validate_feature_flags_get_treatments(  # pylint: disable=invalid-name
+    method_name,
+    feature_flag_names,
+    ):
+    """
+    Check if feature flags is valid for get_treatments.
+
+    :param feature_flags: array of feature flags
+    :type feature_flags: list
+    :return: filtered_feature_flags
     :rtype: tuple
     """
-    if features is None or not isinstance(features, list):
-        _LOGGER.error("%s: feature_names must be a non-empty array.", method_name)
-        return None, None
-    if not features:
-        _LOGGER.error("%s: feature_names must be a non-empty array.", method_name)
-        return None, None
-    filtered_features = set(
-        _remove_empty_spaces(feature, method_name) for feature in features
-        if feature is not None and
-        _check_is_string(feature, 'feature_name', method_name) and
-        _check_string_not_empty(feature, 'feature_name', method_name)
-    )
-    if not filtered_features:
-        _LOGGER.error("%s: feature_names must be a non-empty array.", method_name)
-        return None, None
+    if not _check_feature_flag_instance(feature_flag_names, method_name):
+        return None
 
-    if not should_validate_existance:
-        return filtered_features, []
+    filtered_feature_flags = _get_filtered_feature_flag(feature_flag_names, method_name)
+    if not filtered_feature_flags:
+        _LOGGER.error("%s: feature flag names must be a non-empty array.", method_name)
+        return None
 
-    valid_missing_features = set(f for f in filtered_features if split_storage.get(f) is None)
-    for missing_feature in valid_missing_features:
-        _LOGGER.warning(
-            "%s: you passed \"%s\" that does not exist in this environment, "
-            "please double check what Splits exist in the web console.",
-            method_name,
-            missing_feature
-        )
-    return filtered_features - valid_missing_features, valid_missing_features
+    valid_feature_flags = []
+    for ff in filtered_feature_flags:
+        ff = _remove_empty_spaces(ff, 'feature flag name', method_name)
+        valid_feature_flags.append(ff)
+    return valid_feature_flags
 
-
-def generate_control_treatments(features, method_name):
+def generate_control_treatments(feature_flags):
     """
-    Generate valid features to control.
+    Generate valid feature flags to control.
 
-    :param features: array of features
-    :type features: list
+    :param feature_flags: array of feature flags
+    :type feature_flags: list
     :return: dict
     :rtype: dict|None
     """
-    return {feature: (CONTROL, None) for feature in validate_features_get_treatments(method_name, features)[0]}
+    if not isinstance(feature_flags, list):
+        return {}
+
+    to_return = {}
+    for feature_flag in feature_flags:
+        if isinstance(feature_flag, str) and len(feature_flag.strip())> 0:
+            to_return[feature_flag] = (CONTROL, None)
+    return to_return
 
 
 def validate_attributes(attributes, method_name):
@@ -437,36 +532,49 @@ def validate_attributes(attributes, method_name):
     """
     if attributes is None:
         return True
+
     if not isinstance(attributes, dict):
         _LOGGER.error('%s: attributes must be of type dictionary.', method_name)
         return False
+
     return True
 
+def validate_evaluation_options(evaluation_options, method_name):
+    if evaluation_options == None:
+        return None
+    
+    if not isinstance(evaluation_options, client.EvaluationOptions):
+        _LOGGER.error("%s: evaluation options should be an instance of EvaluationOptions. Setting its value to None.", method_name)
+        return None
+    
+    return evaluation_options    
 
 class _ApiLogFilter(logging.Filter):  # pylint: disable=too-few-public-methods
     def filter(self, record):
         return record.name not in ('SegmentsAPI', 'HttpClient')
-        
 
-def validate_factory_instantiation(apikey):
+
+def validate_factory_instantiation(sdk_key):
     """
     Check if the factory if being instantiated with the appropriate arguments.
 
-    :param apikey: str
-    :type apikey: str
+    :param sdk_key: str
+    :type sdk_key: str
     :return: bool
     :rtype: True|False
     """
-    if apikey == 'localhost':
+    if sdk_key == 'localhost':
         return True
-    if (not _check_not_null(apikey, 'apikey', 'factory_instantiation')) or \
-       (not _check_is_string(apikey, 'apikey', 'factory_instantiation')) or \
-       (not _check_string_not_empty(apikey, 'apikey', 'factory_instantiation')):
+
+    if (not _check_not_null(sdk_key, 'sdk_key', 'factory_instantiation')) or \
+       (not _check_is_string(sdk_key, 'sdk_key', 'factory_instantiation')) or \
+       (not _check_string_not_empty(sdk_key, 'sdk_key', 'factory_instantiation')):
         return False
+
     return True
 
 
-def valid_properties(properties):
+def valid_properties(properties, source):
     """
     Check if properties is a valid dict and returns the properties
     that will be sent to the track method, avoiding unexpected types.
@@ -480,8 +588,9 @@ def valid_properties(properties):
 
     if properties is None:
         return True, None, size
+
     if not isinstance(properties, dict):
-        _LOGGER.error('track: properties must be of type dictionary.')
+        _LOGGER.error('%s: properties must be of type dictionary.', source)
         return False, None, 0
 
     valid_properties = dict()
@@ -496,9 +605,8 @@ def valid_properties(properties):
         if element is None:
             continue
 
-        if not isinstance(element, str) and not isinstance(element, Number) \
-           and not isinstance(element, bool):
-            _LOGGER.warning('Property %s is of invalid type. Setting value to None', element)
+        if not _check_element_type(element):
+            _LOGGER.warning('%s: Property %s is of invalid type. Setting value to None', source, element)
             element = None
 
         valid_properties[property] = element
@@ -508,12 +616,99 @@ def valid_properties(properties):
 
         if size > MAX_PROPERTIES_LENGTH_BYTES:
             _LOGGER.error(
-                'The maximum size allowed for the properties is 32768 bytes. ' +
-                'Current one is ' + str(size) + ' bytes. Event not queued'
-            )
+                '%s: The maximum size allowed for the properties is 32768 bytes. ' +
+                'Current one is ' + str(size) + ' bytes. Event not queued', source)
             return False, None, size
 
     if len(valid_properties.keys()) > 300:
-        _LOGGER.warning('Event has more than 300 properties. Some of them will be trimmed' +
-                        ' when processed')
+        _LOGGER.warning('%s: Event has more than 300 properties. Some of them will be trimmed' +
+                        ' when processed', source)
     return True, valid_properties if len(valid_properties) else None, size
+
+def _check_element_type(element):
+    if not isinstance(element, str) and not isinstance(element, Number) \
+        and not isinstance(element, bool):
+            return False
+    
+    return True
+    
+def validate_pluggable_adapter(config):
+    """
+    Check if pluggable adapter contains the expected method signature
+
+    :param config: config parameters
+    :type config: Dict
+
+    :return: True if no issue found otherwise False
+    :rtype: bool
+    """
+    if config.get('storageType') != 'pluggable':
+        return True
+
+    if config.get('storageWrapper') is None:
+        _LOGGER.error("Expecting pluggable storage `wrapper` in options, but no valid wrapper instance was provided.")
+        return False
+
+    if config.get('storagePrefix') is not None:
+        if not isinstance(config.get('storagePrefix'), str):
+            _LOGGER.error("Pluggable storage prefix should be string type only")
+            return False
+
+    pluggable_adapter = config.get('storageWrapper')
+    if not isinstance(pluggable_adapter, object):
+        _LOGGER.error("Pluggable storage instance is not inherted from object class")
+        return False
+
+    expected_methods = {'get': 1, 'get_items': 1, 'get_many': 1, 'set': 2, 'push_items': 2,
+                        'delete': 1, 'increment': 2, 'decrement': 2, 'get_keys_by_prefix': 1,
+                        'get_many': 1, 'add_items' : 2, 'remove_items': 2, 'item_contains': 2,
+                        'get_items_count': 1, 'expire': 2}
+    methods = inspect.getmembers(pluggable_adapter, predicate=inspect.ismethod)
+    for exp_method in expected_methods:
+        method_found = False
+        get_method_args = set()
+        for method in methods:
+            if exp_method == method[0]:
+                method_found = True
+                get_method_args = inspect.signature(method[1]).parameters
+                break
+
+        if not method_found:
+            _LOGGER.error("Pluggable adapter does not have required method: %s" % exp_method)
+            return False
+
+        if len(get_method_args) < expected_methods[exp_method]:
+            _LOGGER.error("Pluggable adapter method %s has less than required arguments count: %s : " % (exp_method, len(get_method_args)))
+            return False
+
+    return True
+
+def validate_flag_sets(flag_sets, method_name):
+    """
+    Validate flag sets list
+    :param flag_set: list of flag sets
+    :type flag_set: list[str]
+    :returns: Sanitized and sorted flag sets
+    :rtype: list[str]
+    """
+    if not isinstance(flag_sets, list):
+        _LOGGER.warning("%s: flag sets parameter type should be list object, parameter is discarded", method_name)
+        return []
+
+    sanitized_flag_sets = set()
+    for flag_set in flag_sets:
+        if not _check_not_null(flag_set, 'flag set', method_name):
+            continue
+
+        if not _check_is_string(flag_set, 'flag set', method_name):
+            continue
+
+        flag_set = _remove_empty_spaces(flag_set, 'flag set', method_name)
+        flag_set = _convert_str_to_lower(flag_set, 'flag set', method_name)
+
+        if not _check_string_matches(flag_set, method_name, _FLAG_SETS_REGEX, 'a flag set', 50):
+            continue
+
+        sanitized_flag_sets.add(flag_set)
+
+    return list(sanitized_flag_sets)
