@@ -12,7 +12,16 @@ Impression = namedtuple(
         'change_number',
         'bucketing_key',
         'time',
-        'previous_time'
+        'previous_time',
+        'properties'
+    ]
+)
+
+ImpressionDecorated = namedtuple(
+    'ImpressionDecorated',
+    [
+        'Impression',
+        'disabled'
     ]
 )
 
@@ -52,3 +61,8 @@ class Label(object):  # pylint: disable=too-few-public-methods
     # Treatment: control
     # Label: not ready
     NOT_READY = 'not ready'
+
+    # Condition: Prerequisites not met
+    # Treatment: Default treatment
+    # Label: prerequisites not met
+    PREREQUISITES_NOT_MET = "prerequisites not met"
