@@ -1,15 +1,67 @@
 """Splits module."""
 from enum import Enum
 from collections import namedtuple
+import logging
 
+from splitio.models import MatcherNotFoundException
 from splitio.models.grammar import condition
 
+_LOGGER = logging.getLogger(__name__)
 
 SplitView = namedtuple(
     'SplitView',
-    ['name', 'traffic_type', 'killed', 'treatments', 'change_number', 'configs']
+    ['name', 'traffic_type', 'killed', 'treatments', 'change_number', 'configs', 'default_treatment', 'sets', 'impressions_disabled', 'prerequisites']
 )
 
+_DEFAULT_CONDITIONS_TEMPLATE =   {
+    "conditionType": "ROLLOUT",
+    "matcherGroup": {
+        "combiner": "AND",
+        "matchers": [
+        {
+            "keySelector": None,
+            "matcherType": "ALL_KEYS",
+            "negate": False,
+            "userDefinedSegmentMatcherData": None,
+            "whitelistMatcherData": None,
+            "unaryNumericMatcherData": None,
+            "betweenMatcherData": None,
+            "dependencyMatcherData": None,
+            "booleanMatcherData": None,
+            "stringMatcherData": None
+        }]
+    },
+    "partitions": [
+        {
+        "treatment": "control",
+        "size": 100
+        }
+    ],
+    "label": "targeting rule type unsupported by sdk"
+}
+
+class Prerequisites(object):
+    """Prerequisites."""
+    def __init__(self, feature_flag_name, treatments):
+        self._feature_flag_name = feature_flag_name
+        self._treatments = treatments
+    
+    @property
+    def feature_flag_name(self):
+        """Return featur eflag name."""
+        return self._feature_flag_name
+
+    @property
+    def treatments(self):
+        """Return treatments."""
+        return self._treatments
+    
+    def to_json(self):
+        to_return = []
+        for feature_flag_name in self._feature_flag_name:
+            to_return.append({"n": feature_flag_name, "ts": [treatment for treatment in self._treatments]})
+        
+        return to_return
 
 class Status(Enum):
     """Split status."""
@@ -41,7 +93,10 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
             algo=None,
             traffic_allocation=None,
             traffic_allocation_seed=None,
-            configurations=None
+            configurations=None,
+            sets=None,
+            impressions_disabled=None,
+            prerequisites = None
     ):
         """
         Class constructor.
@@ -62,6 +117,12 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
         :type traffic_allocation: int
         :pram traffic_allocation_seed: Seed used to hash traffic allocation.
         :type traffic_allocation_seed: int
+        :pram sets: list of flag sets
+        :type sets: list
+        :pram impressions_disabled: track impressions flag
+        :type impressions_disabled: boolean
+        :pram prerequisites: prerequisites
+        :type prerequisites: List of Preqreuisites
         """
         self._name = name
         self._seed = seed
@@ -90,6 +151,9 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
             self._algo = HashAlgorithm.LEGACY
 
         self._configurations = configurations
+        self._sets = set(sets) if sets is not None else set()
+        self._impressions_disabled = impressions_disabled if impressions_disabled is not None else False
+        self._prerequisites = prerequisites if prerequisites is not None else []
 
     @property
     def name(self):
@@ -146,6 +210,21 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
         """Return the traffic allocation seed of the split."""
         return self._traffic_allocation_seed
 
+    @property
+    def sets(self):
+        """Return the flag sets of the split."""
+        return self._sets
+
+    @property
+    def impressions_disabled(self):
+        """Return impressions_disabled of the split."""
+        return self._impressions_disabled
+    
+    @property
+    def prerequisites(self):
+        """Return prerequisites of the split."""
+        return self._prerequisites
+
     def get_configurations_for(self, treatment):
         """Return the mapping of treatments to configurations."""
         return self._configurations.get(treatment) if self._configurations else None
@@ -173,7 +252,10 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
             'defaultTreatment': self.default_treatment,
             'algo': self.algo.value,
             'conditions': [c.to_json() for c in self.conditions],
-            'configurations': self._configurations
+            'configurations': self._configurations,
+            'sets': list(self._sets),
+            'impressionsDisabled': self._impressions_disabled,
+            'prerequisites': [prerequisite.to_json() for prerequisite in self._prerequisites]
         }
 
     def to_split_view(self):
@@ -189,7 +271,11 @@ class Split(object):  # pylint: disable=too-many-instance-attributes
             self.killed,
             list(set(part.treatment for cond in self.conditions for part in cond.partitions)),
             self.change_number,
-            self._configurations if self._configurations is not None else {}
+            self._configurations if self._configurations is not None else {},
+            self._default_treatment,
+            list(self._sets) if self._sets is not None else [],
+            self._impressions_disabled,
+            self._prerequisites
         )
 
     def local_kill(self, default_treatment, change_number):
@@ -226,6 +312,12 @@ def from_raw(raw_split):
     :return: A parsed Split object capable of performing evaluations.
     :rtype: Split
     """
+    try:
+        conditions = [condition.from_raw(c) for c in raw_split['conditions']]
+    except MatcherNotFoundException as e:
+        _LOGGER.error(str(e))
+        _LOGGER.debug("Using default conditions template for feature flag: %s", raw_split['name'])
+        conditions = [condition.from_raw(_DEFAULT_CONDITIONS_TEMPLATE)]
     return Split(
         raw_split['name'],
         raw_split['seed'],
@@ -234,9 +326,19 @@ def from_raw(raw_split):
         raw_split['trafficTypeName'],
         raw_split['status'],
         raw_split['changeNumber'],
-        [condition.from_raw(c) for c in raw_split['conditions']],
+        conditions,
         raw_split.get('algo'),
         traffic_allocation=raw_split.get('trafficAllocation'),
         traffic_allocation_seed=raw_split.get('trafficAllocationSeed'),
-        configurations=raw_split.get('configurations')
+        configurations=raw_split.get('configurations'),
+        sets=set(raw_split.get('sets')) if raw_split.get('sets') is not None else [],
+        impressions_disabled=raw_split.get('impressionsDisabled') if raw_split.get('impressionsDisabled') is not None else False,
+        prerequisites=from_raw_prerequisites(raw_split.get('prerequisites')) if raw_split.get('prerequisites') is not None else []
     )
+
+def from_raw_prerequisites(raw_prerequisites):
+    to_return = []
+    for prerequisite in raw_prerequisites:
+        to_return.append(Prerequisites(prerequisite['n'], prerequisite['ts']))
+    
+    return to_return
\ No newline at end of file
